@@ -36,8 +36,8 @@ import usagestats
 import yaml
 
 from .utils import iteritems, itervalues, unicode_, stderr, UniqueNames, \
-    escape, CommonEqualityMixin, optional_return_type, hsize, join_root, \
-    copyfile
+    escape, CommonEqualityMixin, optional_return_type, isodatetime, hsize, \
+    join_root, copyfile
 
 
 FILE_READ = 0x01
@@ -489,7 +489,7 @@ version: "{format!s}"
                     else ''),
            version=escape(reprozip_version),
            format='0.8',
-           date=datetime.now().isoformat(),
+           date=isodatetime(),
            what=("# It was generated by the packer and you shouldn't need to "
                  "edit it" if canonical
                  else "# You might want to edit this file before running the "
@@ -554,8 +554,8 @@ other_files:
 
 # If you want to include additional files in the pack, you can list additional
 # patterns of files that will be included
+additional_patterns:
 # Example:
-#additional_patterns:
 #  - /etc/apache2/**  # Everything under apache2/
 #  - /var/log/apache2/*.log  # Log files directly under apache2/
 #  - /var/lib/lxc/*/rootfs/home/**/*.py  # All Python files of all users in
