@@ -88,22 +88,22 @@ class Run(object):
             json_process = self.processes[0].json()
             for process in self.processes:
                 prog_map[process] = json_process
-            return [json_process]
+            processes = [json_process]
         else:
-            run = []
+            processes = []
             process_idx_map = {}
             for process in self.processes:
                 if level_processes == LVL_PROC_THREAD or not process.thread:
-                    process_idx_map[process] = len(run)
+                    process_idx_map[process] = len(processes)
                     json_process = process.json(process_idx_map)
                     prog_map[process] = json_process
-                    run.append(json_process)
+                    processes.append(json_process)
                 else:
                     p_process = process
                     while p_process.thread:
                         p_process = p_process.parent
                     prog_map[process] = prog_map[p_process]
-            return run
+        return {'name': self.name, 'processes': processes}
 
 
 class Process(object):
@@ -112,19 +112,21 @@ class Process(object):
     _id_gen = 0
 
     def __init__(self, pid, run, parent, timestamp, thread, acted, binary,
-                 created):
+                 argv, created):
         self.id = Process._id_gen
         Process._id_gen += 1
         self.pid = pid
         self.run = run
         self.parent = parent
         self.timestamp = timestamp
-        self.thread = thread
+        self.thread = bool(thread)
         # Whether that process has done something yet. If it execve()s and
         # hasn't done anything since it forked, no need for it to appear
         self.acted = acted
         # Executable file
         self.binary = binary
+        # Command-line if this was created by an exec
+        self.argv = argv
         # How was this process created, one of the C_* constants
         self.created = created
 
@@ -176,7 +178,9 @@ class Process(object):
         else:
             parent = None
         return {'name': name, 'parent': parent, 'reads': [], 'writes': [],
-                'long_name': long_name, 'description': description}
+                'long_name': long_name, 'description': description,
+                'argv': self.argv, 'is_thread': self.thread,
+                'start_time': self.timestamp}
 
 
 class Package(object):
@@ -217,6 +221,12 @@ class Package(object):
         else:
             return '"%s"' % escape(unicode_(f))
 
+    def json_endpoint(self, f, level_pkgs):
+        if level_pkgs == LVL_PKG_PACKAGE:
+            return self.name
+        else:
+            return unicode_(f)
+
     def json(self, level_pkgs):
         if level_pkgs == LVL_PKG_PACKAGE:
             logging.critical("JSON output doesn't support --packages package")
@@ -354,6 +364,10 @@ def read_events(database, all_forks, has_thread_flag):
                 runs.append(run)
                 parent = None
                 binary = None
+            if r_parent is not None:
+                argv = processes[r_parent].argv
+            else:
+                argv = None
             process = Process(r_id,
                               run,
                               parent,
@@ -361,6 +375,7 @@ def read_events(database, all_forks, has_thread_flag):
                               r_thread,
                               False,
                               binary,
+                              argv,
                               C_INITIAL if r_parent is None else C_FORK)
             processes[r_id] = process
             all_programs.append(process)
@@ -390,6 +405,7 @@ def read_events(database, all_forks, has_thread_flag):
                 process.binary = r_name
                 process.created = C_FORKEXEC
                 process.acted = True
+                process.argv = argv
             else:
                 process = Process(process.pid,
                                   run,
@@ -398,6 +414,7 @@ def read_events(database, all_forks, has_thread_flag):
                                   False,
                                   True,         # Hides exec only once
                                   r_name,
+                                  argv,
                                   C_EXEC)
                 all_programs.append(process)
                 processes[r_process] = process
@@ -424,7 +441,8 @@ def format_argv(argv):
 def generate(target, configfile, database, all_forks=False, graph_format='dot',
              level_pkgs='file', level_processes='thread',
              level_other_files='all',
-             regex_filters=None, regex_replaces=None, aggregates=None):
+             regex_filters=None, regex_includes=None,
+             regex_replaces=None, aggregates=None):
     """Main function for the graph subcommand.
     """
     try:
@@ -437,6 +455,10 @@ def generate(target, configfile, database, all_forks=False, graph_format='dot',
     level_pkgs, level_processes, level_other_files, file_depth = \
         parse_levels(level_pkgs, level_processes, level_other_files)
 
+    if target.exists():
+        logging.critical("Output file %s exists", target)
+        sys.exit(1)
+
     # Reads package ownership from the configuration
     if not configfile.is_file():
         logging.critical("Configuration file does not exist!\n"
@@ -445,8 +467,9 @@ def generate(target, configfile, database, all_forks=False, graph_format='dot',
                          "alternate location.")
         sys.exit(1)
     config = load_config(configfile, canonical=False)
-    inputs_outputs = dict((f.path, n)
-                          for n, f in iteritems(config.inputs_outputs))
+    inputs_outputs = config.inputs_outputs
+    inputs_outputs_map = dict((f.path, n)
+                              for n, f in iteritems(config.inputs_outputs))
     has_thread_flag = config.format_version >= LooseVersion('0.7')
 
     runs, files, edges = read_events(database, all_forks,
@@ -463,11 +486,16 @@ def generate(target, configfile, database, all_forks=False, graph_format='dot',
     # Apply regexes
     ignore = [lambda path, r=re.compile(p): r.search(path) is not None
               for p in regex_filters or []]
+    include = [lambda path, r=re.compile(p): r.search(path) is not None
+               for p in regex_includes or []]
     replace = [lambda path, r=re.compile(p): r.sub(repl, path)
                for p, repl in regex_replaces or []]
 
     def filefilter(path):
         pathuni = unicode_(path)
+        if include and not any(f(pathuni) for f in include):
+            logging.debug("IGN(include) %s", pathuni)
+            return None
         if any(f(pathuni) for f in ignore):
             logging.debug("IGN %s", pathuni)
             return None
@@ -537,7 +565,8 @@ def generate(target, configfile, database, all_forks=False, graph_format='dot',
                            for prog, f, mode, argv in edges)
     else:
         if level_other_files == LVL_OTHER_IO:
-            other_files = set(f for f in other_files if f in inputs_outputs)
+            other_files = set(f
+                              for f in other_files if f in inputs_outputs_map)
             edges = [(prog, f, mode, argv)
                      for prog, f, mode, argv in edges
                      if f in package_map or f in other_files]
@@ -548,7 +577,8 @@ def generate(target, configfile, database, all_forks=False, graph_format='dot',
                      if f in package_map]
 
     args = (target, runs, packages, other_files, package_map, edges,
-            inputs_outputs, level_pkgs, level_processes, level_other_files)
+            inputs_outputs, inputs_outputs_map,
+            level_pkgs, level_processes, level_other_files)
     if graph_format == FORMAT_DOT:
         graph_dot(*args)
     elif graph_format == FORMAT_JSON:
@@ -558,13 +588,14 @@ def generate(target, configfile, database, all_forks=False, graph_format='dot',
 
 
 def graph_dot(target, runs, packages, other_files, package_map, edges,
-              inputs_outputs, level_pkgs, level_processes, level_other_files):
+              inputs_outputs, inputs_outputs_map,
+              level_pkgs, level_processes, level_other_files):
     """Writes a GraphViz DOT file from the collected information.
     """
     with target.open('w', encoding='utf-8', newline='\n') as fp:
-        fp.write('digraph G {\n    /* programs */\n'
+        fp.write('digraph G {\n    rankdir=LR;\n\n    /* programs */\n'
                  '    node [shape=box fontcolor=white '
-                 'fillcolor=black style=filled];\n')
+                 'fillcolor=black style="filled,rounded"];\n')
 
         # Programs
         logging.info("Writing programs...")
@@ -587,11 +618,11 @@ def graph_dot(target, runs, packages, other_files, package_map, edges,
         # Other files
         logging.info("Writing other files...")
         for fi in sorted(other_files):
-            if fi in inputs_outputs:
+            if fi in inputs_outputs_map:
                 fp.write('    "%(path)s" [fillcolor="#A3B4E0", '
                          'label="%(name)s\\n%(path)s"];\n' %
                          {'path': escape(unicode_(fi)),
-                          'name': inputs_outputs[fi]})
+                          'name': inputs_outputs_map[fi]})
             else:
                 fp.write('    "%s";\n' % escape(unicode_(fi)))
 
@@ -630,11 +661,15 @@ def graph_dot(target, runs, packages, other_files, package_map, edges,
 
 
 def graph_json(target, runs, packages, other_files, package_map, edges,
-               inputs_outputs, level_pkgs, level_processes, level_other_files):
+               inputs_outputs, inputs_outputs_map,
+               level_pkgs, level_processes, level_other_files):
     """Writes a JSON file suitable for further processing.
     """
     # Packages
-    json_packages = [pkg.json(level_pkgs) for pkg in packages]
+    if level_pkgs in (LVL_PKG_IGNORE, LVL_PKG_DROP):
+        json_packages = []
+    else:
+        json_packages = [pkg.json(level_pkgs) for pkg in packages]
 
     # Other files
     json_other_files = [unicode_(fi) for fi in sorted(other_files)]
@@ -644,15 +679,27 @@ def graph_json(target, runs, packages, other_files, package_map, edges,
     json_runs = [run.json(prog_map, level_processes) for run in runs]
 
     # Connect edges
-    for prog, f, mode, argv in edges:
-        what = unicode_(f)
+    done_edges = set()
+    for prog, fi, mode, argv in edges:
+        endp_prog = prog_map[prog]
+        if fi in package_map:
+            if level_pkgs == LVL_PKG_DROP:
+                continue
+            endp_file = package_map[fi].json_endpoint(fi, level_pkgs)
+            e = endp_prog['name'], endp_file, mode
+            if e in done_edges:
+                continue
+            else:
+                done_edges.add(e)
+        else:
+            endp_file = unicode_(fi)
         if mode is None:
-            prog_map[prog]['reads'].append(what)
+            endp_prog['reads'].append(endp_file)
             # TODO: argv?
         elif mode & FILE_WRITE:
-            prog_map[prog]['writes'].append(what)
+            endp_prog['writes'].append(endp_file)
         elif mode & FILE_READ:
-            prog_map[prog]['reads'].append(what)
+            endp_prog['reads'].append(endp_file)
 
     json_other_files.sort()
 
@@ -664,7 +711,12 @@ def graph_json(target, runs, packages, other_files, package_map, edges,
         json.dump({'packages': sorted(json_packages,
                                       key=lambda p: p['name']),
                    'other_files': json_other_files,
-                   'runs': json_runs},
+                   'runs': json_runs,
+                   'inputs_outputs': [
+                       {'name': k, 'path': unicode_(v.path),
+                        'read_by_runs': v.read_runs,
+                        'written_by_runs': v.write_runs}
+                       for k, v in sorted(iteritems(inputs_outputs))]},
                   fp,
                   ensure_ascii=False,
                   indent=2,
@@ -682,7 +734,8 @@ def graph(args):
     def call_generate(args, config, trace):
         generate(Path(args.target[0]), config, trace, args.all_forks,
                  args.format, args.packages, args.processes, args.otherfiles,
-                 args.regex_filter, args.regex_replace, args.aggregate)
+                 args.regex_filter, args.regex_include,
+                 args.regex_replace, args.aggregate)
 
     if args.pack is not None:
         rpz_pack = RPZPack(args.pack)
@@ -729,8 +782,12 @@ def setup(parser, **kwargs):
                         "'io' or 'no' (default: 'all')")
     parser.add_argument('--aggregate', action='append',
                         help="Aggregate all files under this path")
+    parser.add_argument('--regex-include', action='append',
+                        help="Glob patterns of files to include (checked "
+                             "before --regex-filter)")
     parser.add_argument('--regex-filter', action='append',
-                        help="Glob patterns of files to ignore")
+                        help="Glob patterns of files to ignore (checked "
+                             "after --regex-include)")
     parser.add_argument('--regex-replace', action='append', nargs=2,
                         help="Apply regular expression replacement to files")
     parser.add_argument('--dot', action='store_const', dest='format',
